package com.medias.xml.metadata;


import com.medias.ConversionException;
import com.medias.utils.Entity;
import com.medias.xml.XMLBuilder;
import com.medias.xml.hierarchie.Hierarchie;
import org.jdom.Element;
import org.jdom.JDOMException;

import java.io.IOException;
import java.util.Iterator;


/**
 * @author combaz
 * Created on 10 févr. 2005
 * 
 */
public class Metadata implements Entity
{
	
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    public static final int CONTACT_OK   = 0;
	public static final int CONTACT_PI   = 1;
	public static final int CONTACT_TECH = 2;
	public static final int CONTACT_WRONG = 3;
	
	private String absolutePath;
	private String relativePath;
	private String xmlFileName;
	
	private Presentation pres;
	private Description desc;
	private Contenu contenuPrevu;
	private Integration integ;
//	private Contenu contenuActuel;
	
	public Metadata() {
		super();
		this.absolutePath = null;
		this.relativePath = null;
		this.xmlFileName = null;
		this.pres = new Presentation();
		this.desc = new Description();
		this.contenuPrevu = new Contenu();
		this.integ = new Integration();
//		this.contenuActuel = new Contenu();
	}
	
	public Metadata(String path) throws IOException, JDOMException, ConversionException {
		this.pres = new Presentation();
		this.desc = new Description();
		this.contenuPrevu = new Contenu();
		this.integ = new Integration();
//		this.contenuActuel = new Contenu();

		this.setAbsolutePath(path.substring(0,path.lastIndexOf("/")));

		XMLBuilder builder = new XMLBuilder();
		builder.readXml(path);
		this.fromXmlElement(builder.getRoot());
		this.setRelativePath(this.getPres().getCat() + "/" + this.getPres().getNom());
		this.setXmlFileName(this.pres.getNom() + "_metadata.com.medias.xml");
	}
	

	/**
	 * @return Returns the contenuActuel.
	 */
//	public Contenu getContenuActuel() {
//		return contenuActuel;
//	}
//	/**
//	 * @param contenuActuel The contenuActuel to set.
//	 */
//	public void setContenuActuel(Contenu contenuActuel) {
//		this.contenuActuel = contenuActuel;
//	}
	
	/**
	 * @return Returns the contenuPrevu.
	 */
	public Contenu getContenuPrevu() {
		return contenuPrevu;
	}
	
	/**
	 * @param contenuPrevu The contenuPrevu to set.
	 */
	public void setContenuPrevu(Contenu contenuPrevu) {
		this.contenuPrevu = contenuPrevu;
	}
	
	/**
	 * @return Returns the desc.
	 */
	public Description getDesc() {
		return desc;
	}
	
	/**
	 * @param desc The desc to set.
	 */
	public void setDesc(Description desc) {
		this.desc = desc;
	}
	
	/**
	 * @return Returns the integ.
	 */
	public Integration getInteg() {
		return integ;
	}
	
	/**
	 * @param integ The integ to set.
	 */
	public void setInteg(Integration integ) {
		this.integ = integ;
	}
	
	/**
	 * @return Returns the pres.
	 */
	public Presentation getPres() {
		return pres;
	}
	
	/**
	 * @param pres The pres to set.
	 */
	public void setPres(Presentation pres) {
		this.pres = pres;
		this.updateContacts();
	}
	
	/**
	 * @return Returns the absolutePath.
	 */
	public String getAbsolutePath() {
		return absolutePath;
	}
	
	/**
	 * @param absolutePath The absolutePath to set.
	 */
	public void setAbsolutePath(String absolutePath) {
		this.absolutePath = absolutePath;
	}
	
	/**
	 * @return Returns the relativePath.
	 */
	public String getRelativePath() {
		this.relativePath = this.pres.getCat() + "/" + this.pres.getNom();
		return relativePath;
	}
	/**
	 * @param relativePath The relativePath to set.
	 */
	public void setRelativePath(String relativePath) {
		this.relativePath = relativePath;
	}
	/**
	 * @return Returns the xmlFileName.
	 */
	public String getXmlFileName() {
		return xmlFileName;
	}
	/**
	 * @param xmlFileName The xmlFileName to set.
	 */
	public void setXmlFileName(String xmlFileName) {
		this.xmlFileName = xmlFileName;
	}
	
	/**
	 * Sets the path of the com.medias.xml file where metadata are written
	 * @param uploadroot The root directory where data and metadata are saved.
	 */
	public void setXmlFile(String uploadroot) {
		this.setRelativePath(this.pres.getCat() + "/" + this.pres.getNom());
		this.setAbsolutePath(uploadroot + "/" + this.relativePath);
		this.setXmlFileName(this.pres.getNom() + "_metadata.com.medias.xml");
	}
	
	/**
	 * Permet de créer un nouveau jeu de données :
	 * <ul>
	 * 		<li>definit l'endroit ou le fichier com.medias.xml de metadonnes sera stocke</li>
	 * 		<li>met a jour le fichier de description de l'arborescence</li>
	 * 		<li>cree le dossier correspondant au nouveau jeu dans le site de depot</li>
	 * 		<li>cree le fichier com.medias.xml de metadonnees et le sauve au bon endroit.</li>
	 * 		<li>met à jour la vue représentant l'arborescence.</li>
	 * </ul>
	 * @param tree
	 * @throws IOException
	 * @throws JDOMException
	 */
	public void createNewSet(Hierarchie tree) throws Exception {
		this.setXmlFile(tree.getDataRoot());
		tree.saveMetadata(this.relativePath,this.xmlFileName);
		this.createXmlFile();
		tree.createTreeView();
	}
	
	/**
	 * Cree le fichier com.medias.xml de metadonnees et le sauve au bon endroit.
	 * @throws IOException
	 */
	public void createXmlFile() throws IOException {
		Element root = this.toXmlElement("metadata");
		XMLBuilder builder = new XMLBuilder(root);
		builder.writeXml(this.getAbsolutePath() + "/" + this.getXmlFileName());
	}
	
	/**
	 * Synchronise la liste de contacts (enregistrée dans la partie "présentation") avec les responsables
	 * thématique et technique, enregistrés dans la partie "description".
	 * @return
	 */
	public int updateContacts() {
		boolean delete_pi = true;
		boolean delete_tech = true;
		
		int returnValue     = CONTACT_OK;
		
		Iterator<Contact> it = this.getPres().getContactsList().iterator();
		while(it.hasNext()){
			Contact pers = it.next();
			pers.formToModel();
			if(pers.getRole().equals("pi")) {
				if (delete_pi == true) {
					this.desc.setPi(pers);
					delete_pi = false;
				}
				else {
					// on retire de la présentation le contact en double PI / Tech
//					it.remove ();
					returnValue = CONTACT_PI;
					break;
				}			
			}
			else if(pers.getRole().equals("tech")) {
				if (delete_tech == true) {
					this.desc.setTech(pers);
					delete_tech = false;
				}
				else {
					// on retire de la présentation le contact en double PI / Tech
//					it.remove ();
					returnValue = CONTACT_TECH;
					break;
				}
			} 
			// 
			if (!pers.getMail().equals("") && pers.getMdp().equals("")) {
				returnValue = CONTACT_WRONG;
				break;
			}
		}
		
		if (delete_pi)
			this.desc.setPi(new Contact());
		if (delete_tech)
			this.desc.setTech(new Contact());
		return returnValue;
	}
	
	
	/**
	 * Crée un élément JDOM représentant la structure de la métadonnée.
	 * @param title Le nom à donner à l'élément retourné
	 * @return Retourne l'élément JDOM représentant la structure de la métadonnée.
	 */
	public Element toXmlElement (String title) {
		Element p 	= this.pres.toXmlElement ("presentation");
		Element d 	= this.desc.toXmlElement ("description");
		Element cp 	= this.contenuPrevu.toXmlElement ("contenu_prevu");
		Element i 	= this.integ.toXmlElement ("integration");
//		Element ca = this.contenuActuel.toXmlElement("contenu_actuel");
		
		Element metadata = new Element (title);
		metadata.addContent (p);
		metadata.addContent (d);
		metadata.addContent (cp);
		metadata.addContent (i);
//		metadata.addContent(ca);
		return metadata;
	}

	/**
	 * Met à jour tous les champs à partir de l'élément JDOM passé en paramètre.
	 */
	public void fromXmlElement (Element el) throws ConversionException {
		Element p = el.getChild ("presentation");
		Element d = el.getChild ("description");
		Element cp = el.getChild ("contenu_prevu");
		Element i = el.getChild ("integration");
//		Element ca = el.getChild("contenu_actuel");

		if((p == null) || (d == null) || (cp == null) || (i == null))
		        //||(ca==null))
			throw new ConversionException (ConversionException.ERROR_fromXml + "Metadata !");
		
		else {
			this.pres.fromXmlElement (p);
			this.desc.fromXmlElement (d);
			this.contenuPrevu.fromXmlElement (cp);
			this.integ.fromXmlElement (i);
//			this.contenuActuel.fromXmlElement(ca);
		}
	}
}
