package com.medias.xml.metadata;

import com.medias.ConversionException;
import com.medias.utils.Entity;
import org.jdom.Element;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author combaz
 * Created on 10 févr. 2005
 * 
 * Represente la partie "Presentation" des metadonnees.
 */
public class Presentation implements Entity
{
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private String nom;
	private String def;
	private String dest;
	private String cat;
	private ArrayList<Contact> contacts;
	
	public Presentation() {
		super();
		this.nom = null;
		this.def = null;
		this.dest = null;
		this.cat = null;
		this.contacts = new ArrayList<Contact>();
	}
	
	public ArrayList<Contact> getContactsList() {
		return this.contacts;
	}
	/**
	 * @return Returns the contacts.
	 */
	public Object[] getContacts() {
		if(this.contacts.isEmpty()) {
			Contact pers = new Contact();
			this.contacts.add(pers);
		}
		return this.contacts.toArray();
	}
	/**
	 * @param contacts The contacts to set.
	 */
	public void setContacts(Contact[] contacts) {
		this.contacts = new ArrayList<Contact>();
		for(int i=0; i<contacts.length; i++)
			this.contacts.add(contacts[i]);
	}
	/**
	 * @return Returns the cat.
	 */
	public String getCat() {
		return cat;
	}
	/**
	 * @param cat The cat to set.
	 */
	public void setCat(String cat) {
		this.cat = cat;
	}
	/**
	 * @return Returns the def.
	 */
	public String getDef() {
		return def;
	}
	/**
	 * @param def The def to set.
	 */
	public void setDef(String def) {
		this.def = def;
	}
	/**
	 * @return Returns the dest.
	 */
	public String getDest() {
		return dest;
	}
	/**
	 * @param dest The dest to set.
	 */
	public void setDest(String dest) {
		this.dest = dest;
	}
	/**
	 * @return Returns the nom.
	 */
	public String getNom() {
		return nom;
	}
	/**
	 * @param nom The nom to set.
	 */
	public void setNom(String nom) {
		this.nom = nom;
	}
	
	public void addContact() {
		Contact contact = new Contact();
		this.contacts.add(contact);
	}
	
	public void removeContact(Integer idx) {
//	    System.out.println ("PRESENTATION : removeContact (" + idx + ")");
		this.contacts.remove(idx.intValue());
	}
	
	
	public Element toXmlElement(String title) {
		Element n = new Element("nom");
		n.addContent(this.nom);
		Element d = new Element("definition");
		d.addContent(this.def);
		Element de = new Element("destination");
		de.addContent(this.dest);
		Element c = new Element("categorie");
		c.addContent(this.cat);
		Element p = new Element("contacts");
		ArrayList<Element> pl = new ArrayList<Element>();
		for(int i = 0; i<this.contacts.size(); i++) {
			Contact pers = (Contact)(this.contacts.get(i));
			Element elem = pers.toXmlElement("personne");
			pl.add(elem);
		}
		p.addContent(pl);
		
		Element pres = new Element(title);
		pres.addContent(n);
		pres.addContent(d);
		pres.addContent(de);
		pres.addContent(c);
		pres.addContent(p);
		
		return pres;
	}
	
	public void fromXmlElement(Element el) throws ConversionException {
		Element n = el.getChild("nom");
		Element d = el.getChild("definition");
		Element de = el.getChild("destination");
		Element c = el.getChild("categorie");
		Element p = el.getChild("contacts");
		
		if((n==null)||(d==null)||(de==null)||(c==null)||(p==null)) {
			throw new ConversionException(ConversionException.ERROR_fromXml + "Presentation !");
		}
		
		else {
			this.setNom(n.getText());
			this.setDef(d.getText());
			this.setDest(de.getText());
			this.setCat(c.getText());
			List<?> pl = p.getChildren("personne");
			Iterator<?> plit = pl.iterator();
			while(plit.hasNext()) {
				Element elem = (Element) plit.next();
				Contact pers = new Contact();
				pers.fromXmlElement(elem);
				this.contacts.add(pers);
			}
		}
	}

/*	public void reset() {
		this.nom = null;
		this.def = null;
		this.dest = null;
		this.cat = null;
	}*/
}
